@@ -28,12 +28,13 @@ namespace surfaces { class Surface; }
 
 namespace shell
 {
+class Session;
 struct SurfaceCreationParameters;
 
 class SurfaceBuilder
 {
 public:
-    virtual std::weak_ptr<surfaces::Surface> create_surface(SurfaceCreationParameters const& params) = 0;
+    virtual std::weak_ptr<surfaces::Surface> create_surface(Session* session, SurfaceCreationParameters const& params) = 0;
 
     virtual void destroy_surface(std::weak_ptr<surfaces::Surface> const& surface) = 0;
 protected:
