@@ -24,13 +24,13 @@
 
 namespace mir
 {
-namespace events
+namespace frontend
 {
 class EventSink;
 }
-
 namespace shell
 {
+class Session;
 class Surface;
 struct SurfaceCreationParameters;
 
@@ -38,9 +38,10 @@ class SurfaceFactory
 {
 public:
     virtual std::shared_ptr<Surface> create_surface(
+        Session* session,
         SurfaceCreationParameters const& params,
         frontend::SurfaceId id,
-        std::shared_ptr<events::EventSink> const& sink) = 0;
+        std::shared_ptr<frontend::EventSink> const& sink) = 0;
 
 protected:
     virtual ~SurfaceFactory() {}
