@@ -28,15 +28,19 @@ namespace mir
 
 namespace shell
 {
+class Session;
 class SurfaceBuilder;
+class SurfaceConfigurator;
 
 class SurfaceSource : public SurfaceFactory
 {
 public:
-    explicit SurfaceSource(std::shared_ptr<SurfaceBuilder> const& surface_builder);
+    SurfaceSource(std::shared_ptr<SurfaceBuilder> const& surface_builder,
+                  std::shared_ptr<SurfaceConfigurator> const& surface_configurator);
     virtual ~SurfaceSource() {}
 
     std::shared_ptr<Surface> create_surface(
+        Session* session,
         shell::SurfaceCreationParameters const& params,
         frontend::SurfaceId id,
         std::shared_ptr<frontend::EventSink> const& sink);
@@ -47,6 +51,7 @@ protected:
 
 private:
     std::shared_ptr<SurfaceBuilder> const surface_builder;
+    std::shared_ptr<SurfaceConfigurator> const surface_configurator;
 };
 
 }
