@@ -35,10 +35,10 @@ public:
     ~MockCompositingCriteria() noexcept {}
     MOCK_CONST_METHOD0(alpha, float());
     MOCK_CONST_METHOD0(transformation, glm::mat4 const&());
-    MOCK_CONST_METHOD0(should_be_rendered, bool());
+    MOCK_CONST_METHOD1(should_be_rendered_in, bool(geometry::Rectangle const&));
+    MOCK_CONST_METHOD0(shaped, bool());
 };
 
-typedef ::testing::NiceMock<MockCompositingCriteria> StubCompositingCriteria;
 }
 }
 }
