@@ -33,8 +33,9 @@ namespace doubles
 struct MockInputRegistrar : public surfaces::InputRegistrar
 {
     virtual ~MockInputRegistrar() noexcept(true) {}
-    MOCK_METHOD2(input_channel_opened, void(std::shared_ptr<input::InputChannel> const&,
-                                            std::shared_ptr<input::Surface> const&));
+    MOCK_METHOD3(input_channel_opened, void(std::shared_ptr<input::InputChannel> const&,
+                                            std::shared_ptr<input::Surface> const&,
+                                            input::InputReceptionMode mode));
     MOCK_METHOD1(input_channel_closed, void(std::shared_ptr<input::InputChannel> const&));
 };
 
