@@ -31,7 +31,8 @@ namespace doubles
 struct StubInputRegistrar : public surfaces::InputRegistrar
 {
     void input_channel_opened(std::shared_ptr<input::InputChannel> const&,
-                              std::shared_ptr<input::Surface> const&)
+                              std::shared_ptr<input::Surface> const&,
+                              input::InputReceptionMode)
     {
     }
     void input_channel_closed(std::shared_ptr<input::InputChannel> const&)
