@@ -24,7 +24,7 @@
 #include "mir/geometry/dimensions.h"
 #include "mir_toolkit/mir_client_library.h"
 #include "mir_toolkit/common.h"
-#include "mir_toolkit/mir_native_buffer.h"
+#include "mir/graphics/native_buffer.h"
 #include "client_buffer_depository.h"
 #include "mir_wait_handle.h"
 #include "mir_client_surface.h"
@@ -79,9 +79,10 @@ public:
     MirWaitHandle* next_buffer(mir_surface_callback callback, void * context);
     MirWaitHandle* get_create_wait_handle();
 
-    std::shared_ptr<MirNativeBuffer> get_current_buffer_package();
+    MirNativeBuffer* get_current_buffer_package();
     MirPlatformType platform_type();
     std::shared_ptr<mir::client::ClientBuffer> get_current_buffer();
+    uint32_t get_current_buffer_id() const;
     void get_cpu_region(MirGraphicsRegion& region);
     void release_cpu_region();
     EGLNativeWindowType generate_native_window();
@@ -122,7 +123,7 @@ private:
     mir::protobuf::SurfaceSetting configure_result;
 
     // Cache of latest SurfaceSettings returned from the server
-    int attrib_cache[mir_surface_attrib_arraysize_];
+    int attrib_cache[mir_surface_attrib_enum_max_];
 
     std::function<void(MirEvent const*)> handle_event_callback;
     std::shared_ptr<mir::input::receiver::InputReceiverThread> input_thread;
