@@ -19,31 +19,34 @@
 #include "mir/default_server_configuration.h"
 #include "mir/abnormal_exit.h"
 #include "mir/asio_main_loop.h"
+#include "mir/default_pause_resume_listener.h"
 #include "mir/shared_library.h"
 
 #include "mir/options/program_option.h"
-#include "mir/compositor/buffer_allocation_strategy.h"
-#include "mir/compositor/buffer_swapper.h"
 #include "mir/compositor/buffer_stream_factory.h"
 #include "mir/compositor/default_display_buffer_compositor_factory.h"
 #include "mir/compositor/multi_threaded_compositor.h"
-#include "mir/compositor/swapper_factory.h"
 #include "mir/compositor/overlay_renderer.h"
 #include "mir/frontend/protobuf_ipc_factory.h"
 #include "mir/frontend/session_mediator_report.h"
 #include "mir/frontend/null_message_processor_report.h"
 #include "mir/frontend/session_mediator.h"
 #include "mir/frontend/session_authorizer.h"
+#include "mir/frontend/global_event_sender.h"
 #include "mir/frontend/resource_cache.h"
 #include "mir/shell/session_manager.h"
+#include "mir/shell/unauthorized_display_changer.h"
 #include "mir/shell/registration_order_focus_sequence.h"
-#include "mir/shell/single_visibility_focus_mechanism.h"
+#include "mir/shell/default_focus_mechanism.h"
 #include "mir/shell/default_session_container.h"
 #include "mir/shell/consuming_placement_strategy.h"
 #include "mir/shell/organising_surface_factory.h"
 #include "mir/shell/threaded_snapshot_strategy.h"
 #include "mir/shell/graphics_display_layout.h"
+#include "mir/shell/surface_configurator.h"
+#include "mir/shell/broadcasting_session_event_sink.h"
 #include "mir/graphics/cursor.h"
+#include "mir/graphics/nested/host_connection.h"
 #include "mir/shell/null_session_listener.h"
 #include "mir/graphics/display.h"
 #include "mir/shell/gl_pixel_buffer.h"
@@ -56,12 +59,16 @@
 #include "mir/graphics/display_buffer.h"
 #include "mir/graphics/default_display_configuration_policy.h"
 #include "mir/input/cursor_listener.h"
+#include "mir/input/nested_input_configuration.h"
 #include "mir/input/null_input_configuration.h"
 #include "mir/input/null_input_report.h"
 #include "mir/input/display_input_region.h"
-#include "input/android/default_android_input_configuration.h"
-#include "input/android/android_input_manager.h"
-#include "input/android/android_input_targeter.h"
+#include "mir/input/event_filter_chain.h"
+#include "mir/input/nested_input_relay.h"
+#include "mir/input/vt_filter.h"
+#include "mir/input/android/default_android_input_configuration.h"
+#include "mir/input/input_manager.h"
+#include "mir/logging/connector_report.h"
 #include "mir/logging/logger.h"
 #include "mir/logging/input_report.h"
 #include "mir/logging/dumb_console_logger.h"
@@ -72,6 +79,7 @@
 #include "mir/lttng/message_processor_report.h"
 #include "mir/lttng/input_report.h"
 #include "mir/shell/surface_source.h"
+#include "mir/shell/mediating_display_changer.h"
 #include "mir/surfaces/surface_allocator.h"
 #include "mir/surfaces/surface_stack.h"
 #include "mir/surfaces/surface_controller.h"
@@ -79,11 +87,11 @@
 #include "mir/geometry/rectangles.h"
 #include "mir/default_configuration.h"
 #include "mir/graphics/native_platform.h"
+#include "mir/graphics/nested/nested_platform.h"
 
 #include <map>
 
 namespace mc = mir::compositor;
-namespace me = mir::events;
 namespace geom = mir::geometry;
 namespace mf = mir::frontend;
 namespace mg = mir::graphics;
@@ -95,7 +103,6 @@ namespace mia = mi::android;
 
 namespace
 {
-std::initializer_list<std::shared_ptr<mi::EventFilter> const> empty_filter_list{};
 mir::SharedLibrary const* load_library(std::string const& libname);
 }
 
@@ -109,14 +116,14 @@ public:
         std::shared_ptr<mf::SessionMediatorReport> const& sm_report,
         std::shared_ptr<mf::MessageProcessorReport> const& mr_report,
         std::shared_ptr<mg::Platform> const& graphics_platform,
-        std::shared_ptr<mg::Display> const& graphics_display,
-        std::shared_ptr<mc::GraphicBufferAllocator> const& buffer_allocator) :
+        std::shared_ptr<mf::DisplayChanger> const& display_changer,
+        std::shared_ptr<mg::GraphicBufferAllocator> const& buffer_allocator) :
         shell(shell),
         sm_report(sm_report),
         mp_report(mr_report),
         cache(std::make_shared<mf::ResourceCache>()),
         graphics_platform(graphics_platform),
-        graphics_display(graphics_display),
+        display_changer(display_changer),
         buffer_allocator(buffer_allocator)
     {
     }
@@ -127,16 +134,26 @@ private:
     std::shared_ptr<mf::MessageProcessorReport> const mp_report;
     std::shared_ptr<mf::ResourceCache> const cache;
     std::shared_ptr<mg::Platform> const graphics_platform;
-    std::shared_ptr<mg::Display> const graphics_display;
-    std::shared_ptr<mc::GraphicBufferAllocator> const buffer_allocator;
+    std::shared_ptr<mf::DisplayChanger> const display_changer;
+    std::shared_ptr<mg::GraphicBufferAllocator> const buffer_allocator;
 
     virtual std::shared_ptr<mir::protobuf::DisplayServer> make_ipc_server(
-        std::shared_ptr<me::EventSink> const& sink)
+        std::shared_ptr<mf::EventSink> const& sink, bool authorized_to_resize_display)
     {
+        std::shared_ptr<mf::DisplayChanger> changer;
+        if(authorized_to_resize_display)
+        {
+            changer = display_changer; 
+        }
+        else
+        {
+            changer = std::make_shared<msh::UnauthorizedDisplayChanger>(display_changer); 
+        }
+
         return std::make_shared<mf::SessionMediator>(
             shell,
             graphics_platform,
-            graphics_display,
+            changer,
             buffer_allocator,
             sm_report,
             sink,
@@ -154,11 +171,16 @@ private:
     }
 };
 
+char const* const server_socket_opt           = "file";
+char const* const no_server_socket_opt        = "no-file";
 char const* const session_mediator_report_opt = "session-mediator-report";
 char const* const msg_processor_report_opt    = "msg-processor-report";
 char const* const display_report_opt          = "display-report";
 char const* const legacy_input_report_opt     = "legacy-input-report";
+char const* const connector_report_opt        = "connector-report";
 char const* const input_report_opt            = "input-report";
+char const* const host_socket_opt             = "host-socket";
+char const* const standalone_opt              = "standalone";
 
 char const* const glog                 = "glog";
 char const* const glog_stderrthreshold = "glog-stderrthreshold";
@@ -176,7 +198,7 @@ char const* const default_platform_graphics_lib = "libmirplatformgraphics.so";
 
 void parse_arguments(
     boost::program_options::options_description desc,
-    std::shared_ptr<mir::options::ProgramOption> const& options,
+    mir::options::ProgramOption& options,
     int argc,
     char const* argv[])
 {
@@ -187,9 +209,9 @@ void parse_arguments(
         desc.add_options()
             ("help,h", "this help text");
 
-        options->parse_arguments(desc, argc, argv);
+        options.parse_arguments(desc, argc, argv);
 
-        if (options->is_set("help"))
+        if (options.is_set("help"))
         {
             std::ostringstream help_text;
             help_text << desc;
@@ -206,9 +228,16 @@ void parse_arguments(
 
 void parse_environment(
     boost::program_options::options_description& desc,
-    std::shared_ptr<mir::options::ProgramOption> const& options)
+    mir::options::ProgramOption& options)
 {
-    options->parse_environment(desc, "MIR_SERVER_");
+    // If MIR_SERVER_HOST_SOCKET is unset, we want to substitute the value of
+    // MIR_SOCKET.  Do this now, because MIR_SOCKET will get overwritten later.
+    auto host_socket = getenv("MIR_SERVER_HOST_SOCKET");
+    auto mir_socket = getenv("MIR_SOCKET");
+    if (!host_socket && mir_socket)
+        setenv("MIR_SERVER_HOST_SOCKET", mir_socket, 1);
+
+    options.parse_environment(desc, "MIR_SERVER_");
 }
 }
 
@@ -217,19 +246,25 @@ mir::DefaultServerConfiguration::DefaultServerConfiguration(int argc, char const
     argv(argv),
     program_options(std::make_shared<boost::program_options::options_description>(
     "Command-line options.\n"
-    "Environment variables capitalise long form with prefix \"MIR_SERVER_\" and \"_\" in place of \"-\""))
+    "Environment variables capitalise long form with prefix \"MIR_SERVER_\" and \"_\" in place of \"-\"")),
+    default_filter(std::make_shared<mi::VTFilter>())
 {
     namespace po = boost::program_options;
 
     add_options()
-        ("nested-mode", po::value<std::string>(),
-            "Run mir in nested mode. Host socket filename.")
+        (standalone_opt, po::value<bool>(),
+            "Run mir in standalone mode. [bool:default=false]")
+        (host_socket_opt, po::value<std::string>(),
+            "Host socket filename. [string:default={$MIR_SOCKET,$XDG_RUNTIME_DIR/mir_socket}]")
         ("file,f", po::value<std::string>(),
-            "Socket filename")
+            "Socket filename. [string:default=$XDG_RUNTIME_DIR/mir_socket]")
+        (no_server_socket_opt, "Do not provide a socket filename for client connections")
         (platform_graphics_lib, po::value<std::string>(),
-            "Library to use for platform graphics support [default=libmirplatformgraphics.so")
+            "Library to use for platform graphics support [default=libmirplatformgraphics.so]")
         ("enable-input,i", po::value<bool>(),
             "Enable input. [bool:default=true]")
+        (connector_report_opt, po::value<std::string>(),
+            "How to handle the Connector report. [{log,off}:default=off]")
         (display_report_opt, po::value<std::string>(),
             "How to handle the Display report. [{log,off}:default=off]")
         (input_report_opt, po::value<std::string>(),
@@ -258,7 +293,7 @@ mir::DefaultServerConfiguration::DefaultServerConfiguration(int argc, char const
             "directory instead of the default logging directory."
             " [string:default=\"\"]")
         ("ipc-thread-pool", po::value<int>(),
-            "threads in frontend thread pool. [int:default=10]")
+            "threads in frontend thread pool.")
         ("vt", po::value<int>(),
             "VT to run on or 0 to use current. [int:default=0]");
 }
@@ -273,7 +308,20 @@ boost::program_options::options_description_easy_init mir::DefaultServerConfigur
 
 std::string mir::DefaultServerConfiguration::the_socket_file() const
 {
-    return the_options()->get("file", mir::default_server_socket);
+    auto socket_file = the_options()->get(server_socket_opt, mir::default_server_socket);
+
+    // Record this for any children that want to know how to connect to us.
+    // By both listening to this env var on startup and resetting it here,
+    // we make it easier to nest Mir servers.
+    setenv("MIR_SOCKET", socket_file.c_str(), 1);
+
+    return socket_file;
+}
+
+void mir::DefaultServerConfiguration::parse_options(boost::program_options::options_description& options_description, mir::options::ProgramOption& options) const
+{
+    parse_arguments(options_description, options, argc, argv);
+    parse_environment(options_description, options); 
 }
 
 std::shared_ptr<mir::options::Option> mir::DefaultServerConfiguration::the_options() const
@@ -281,10 +329,7 @@ std::shared_ptr<mir::options::Option> mir::DefaultServerConfiguration::the_optio
     if (!options)
     {
         auto options = std::make_shared<mir::options::ProgramOption>();
-
-        parse_arguments(*program_options, options, argc, argv);
-        parse_environment(*program_options, options);
-
+        parse_options(*program_options, *options);
         this->options = options;
     }
     return options;
@@ -310,20 +355,24 @@ std::shared_ptr<mg::DisplayReport> mir::DefaultServerConfiguration::the_display_
 std::shared_ptr<mg::Platform> mir::DefaultServerConfiguration::the_graphics_platform()
 {
     return graphics_platform(
-        [this]()
+        [this]()->std::shared_ptr<mg::Platform>
         {
             auto graphics_lib = load_library(the_options()->get(platform_graphics_lib, default_platform_graphics_lib));
 
-            if (!the_options()->is_set("nested-mode"))
+            // TODO (default-nested): don't fallback to standalone if host socket is unset in 14.04
+            if (the_options()->is_set(standalone_opt) || !the_options()->is_set(host_socket_opt))
             {
                 auto create_platform = graphics_lib->load_function<mg::CreatePlatform>("create_platform");
                 return create_platform(the_options(), the_display_report());
             }
-            else
-            {
-                auto create_native_platform = graphics_lib->load_function<mg::CreateNativePlatform>("create_native_platform");
-                return create_nested_platform(the_options(), create_native_platform());
-            }
+
+            auto create_native_platform = graphics_lib->load_function<mg::CreateNativePlatform>("create_native_platform");
+
+            return std::make_shared<mir::graphics::nested::NestedPlatform>(
+                the_host_connection(),
+                the_nested_input_relay(),
+                the_display_report(),
+                create_native_platform(the_display_report()));
         });
 }
 
@@ -337,16 +386,6 @@ mir::DefaultServerConfiguration::the_buffer_initializer()
         });
 }
 
-std::shared_ptr<mc::BufferAllocationStrategy>
-mir::DefaultServerConfiguration::the_buffer_allocation_strategy()
-{
-    return buffer_allocation_strategy(
-        [this]()
-        {
-            return std::make_shared<mc::SwapperFactory>(the_buffer_allocator());
-        });
-}
-
 std::shared_ptr<mc::RendererFactory> mir::DefaultServerConfiguration::the_renderer_factory()
 {
     return renderer_factory(
@@ -356,6 +395,34 @@ std::shared_ptr<mc::RendererFactory> mir::DefaultServerConfiguration::the_render
         });
 }
 
+std::shared_ptr<msh::MediatingDisplayChanger>
+mir::DefaultServerConfiguration::the_mediating_display_changer()
+{
+    return mediating_display_changer(
+        [this]()
+        {
+            return std::make_shared<msh::MediatingDisplayChanger>(
+                the_display(),
+                the_compositor(),
+                the_display_configuration_policy(),
+                the_shell_session_container(),
+                the_shell_session_event_handler_register());
+        });
+
+}
+
+std::shared_ptr<mf::DisplayChanger>
+mir::DefaultServerConfiguration::the_frontend_display_changer()
+{
+    return the_mediating_display_changer();
+}
+
+std::shared_ptr<mir::DisplayChanger>
+mir::DefaultServerConfiguration::the_display_changer()
+{
+    return the_mediating_display_changer();
+}
+
 std::shared_ptr<msh::SessionContainer>
 mir::DefaultServerConfiguration::the_shell_session_container()
 {
@@ -369,8 +436,7 @@ mir::DefaultServerConfiguration::the_shell_focus_setter()
     return shell_focus_setter(
         [this]
         {
-            return std::make_shared<msh::SingleVisibilityFocusMechanism>(
-                the_shell_session_container(), the_input_targeter());
+            return std::make_shared<msh::DefaultFocusMechanism>(the_input_targeter(), the_surface_controller());
         });
 }
 
@@ -418,6 +484,7 @@ mir::DefaultServerConfiguration::the_session_manager()
                 the_shell_focus_sequence(),
                 the_shell_focus_setter(),
                 the_shell_snapshot_strategy(),
+                the_shell_session_event_sink(),
                 the_shell_session_listener());
         });
 }
@@ -444,22 +511,49 @@ mir::DefaultServerConfiguration::the_shell_snapshot_strategy()
         });
 }
 
+std::shared_ptr<msh::SessionEventSink>
+mir::DefaultServerConfiguration::the_shell_session_event_sink()
+{
+    return the_broadcasting_session_event_sink();
+}
+
+std::shared_ptr<msh::SessionEventHandlerRegister>
+mir::DefaultServerConfiguration::the_shell_session_event_handler_register()
+{
+    return the_broadcasting_session_event_sink();
+}
+
 std::shared_ptr<mf::Shell>
 mir::DefaultServerConfiguration::the_frontend_shell()
 {
     return the_session_manager();
 }
 
+std::shared_ptr<mf::EventSink>
+mir::DefaultServerConfiguration::the_global_event_sink()
+{
+    return global_event_sink(
+        [this]()
+        {
+            return std::make_shared<mf::GlobalEventSender>(the_shell_session_container());
+        }); 
+}
+
 std::shared_ptr<msh::FocusController>
 mir::DefaultServerConfiguration::the_focus_controller()
 {
     return the_session_manager();
 }
 
-std::initializer_list<std::shared_ptr<mi::EventFilter> const>
-mir::DefaultServerConfiguration::the_event_filters()
+std::shared_ptr<mi::CompositeEventFilter>
+mir::DefaultServerConfiguration::the_composite_event_filter()
 {
-    return empty_filter_list;
+    return composite_event_filter(
+        [this]() -> std::shared_ptr<mi::CompositeEventFilter>
+        {
+            std::initializer_list<std::shared_ptr<mi::EventFilter> const> filter_list {default_filter};
+            return std::make_shared<mi::EventFilterChain>(filter_list);
+        });
 }
 
 std::shared_ptr<mi::InputReport>
@@ -515,22 +609,33 @@ mir::DefaultServerConfiguration::the_cursor_listener()
 std::shared_ptr<mi::InputConfiguration>
 mir::DefaultServerConfiguration::the_input_configuration()
 {
-    if (!input_configuration)
+    return input_configuration(
+    [this]() -> std::shared_ptr<mi::InputConfiguration>
     {
-        if (the_options()->get("enable-input", enable_input_default))
+        auto const options = the_options();
+        if (!options->get("enable-input", enable_input_default))
         {
-            input_configuration = std::make_shared<mia::DefaultInputConfiguration>(
-                the_event_filters(),
+            return std::make_shared<mi::NullInputConfiguration>();
+        }
+        // TODO (default-nested): don't fallback to standalone if host socket is unset in 14.04
+        else if (options->is_set(standalone_opt) || !options->is_set(host_socket_opt))
+        {
+            return std::make_shared<mia::DefaultInputConfiguration>(
+                the_composite_event_filter(),
                 the_input_region(),
                 the_cursor_listener(),
                 the_input_report());
         }
         else
         {
-            input_configuration = std::make_shared<mi::NullInputConfiguration>();
+            return std::make_shared<mi::NestedInputConfiguration>(
+                the_nested_input_relay(),
+                the_composite_event_filter(),
+                the_input_region(),
+                the_cursor_listener(),
+                the_input_report());
         }
-    }
-    return input_configuration;
+    });
 }
 
 std::shared_ptr<mi::InputManager>
@@ -545,7 +650,7 @@ mir::DefaultServerConfiguration::the_input_manager()
         });
 }
 
-std::shared_ptr<mc::GraphicBufferAllocator>
+std::shared_ptr<mg::GraphicBufferAllocator>
 mir::DefaultServerConfiguration::the_buffer_allocator()
 {
     return buffer_allocator(
@@ -584,6 +689,25 @@ std::shared_ptr<msh::DisplayLayout> mir::DefaultServerConfiguration::the_shell_d
         });
 }
 
+std::shared_ptr<msh::SurfaceConfigurator> mir::DefaultServerConfiguration::the_shell_surface_configurator()
+{
+    struct DefaultSurfaceConfigurator : public msh::SurfaceConfigurator
+    {
+        int select_attribute_value(msh::Surface const&, MirSurfaceAttrib, int requested_value)
+        {
+            return requested_value;
+        }
+        void attribute_set(msh::Surface const&, MirSurfaceAttrib, int)
+        {
+        }
+    };
+    return shell_surface_configurator(
+        [this]()
+        {
+            return std::make_shared<DefaultSurfaceConfigurator>();
+        });
+}
+
 std::shared_ptr<ms::SurfaceStackModel>
 mir::DefaultServerConfiguration::the_surface_stack_model()
 {
@@ -619,7 +743,7 @@ mir::DefaultServerConfiguration::the_shell_surface_factory()
         [this]()
         {
             auto surface_source = std::make_shared<msh::SurfaceSource>(
-                the_surface_builder());
+                the_surface_builder(), the_shell_surface_configurator());
 
             return std::make_shared<msh::OrganisingSurfaceFactory>(
                 surface_source,
@@ -629,6 +753,12 @@ mir::DefaultServerConfiguration::the_shell_surface_factory()
 
 std::shared_ptr<msh::SurfaceBuilder>
 mir::DefaultServerConfiguration::the_surface_builder()
+{
+    return the_surface_controller();
+}
+
+std::shared_ptr<ms::SurfaceController>
+mir::DefaultServerConfiguration::the_surface_controller()
 {
     return surface_controller(
         [this]()
@@ -670,7 +800,7 @@ mir::DefaultServerConfiguration::the_buffer_stream_factory()
     return buffer_stream_factory(
         [this]()
         {
-            return std::make_shared<mc::BufferStreamFactory>(the_buffer_allocation_strategy());
+            return std::make_shared<mc::BufferStreamFactory>(the_buffer_allocator());
         });
 }
 
@@ -689,8 +819,7 @@ mir::DefaultServerConfiguration::the_compositor()
 std::shared_ptr<mir::frontend::ProtobufIpcFactory>
 mir::DefaultServerConfiguration::the_ipc_factory(
     std::shared_ptr<mf::Shell> const& shell,
-    std::shared_ptr<mg::Display> const& display,
-    std::shared_ptr<mc::GraphicBufferAllocator> const& allocator)
+    std::shared_ptr<mg::GraphicBufferAllocator> const& allocator)
 {
     return ipc_factory(
         [&]()
@@ -700,7 +829,7 @@ mir::DefaultServerConfiguration::the_ipc_factory(
                 the_session_mediator_report(),
                 the_message_processor_report(),
                 the_graphics_platform(),
-                display, allocator);
+                the_frontend_display_changer(), allocator);
         });
 }
 
@@ -713,6 +842,11 @@ mir::DefaultServerConfiguration::the_session_authorizer()
         {
             return true;
         }
+
+        bool configure_display_is_allowed(pid_t /* pid */)
+        {
+            return true;
+        }
     };
     return session_authorizer(
         [&]()
@@ -822,6 +956,16 @@ std::shared_ptr<mir::MainLoop> mir::DefaultServerConfiguration::the_main_loop()
         });
 }
 
+
+std::shared_ptr<mir::PauseResumeListener> mir::DefaultServerConfiguration::the_pause_resume_listener()
+{
+    return pause_resume_listener(
+        []()
+        {
+            return std::make_shared<mir::DefaultPauseResumeListener>();
+        });
+}
+
 std::shared_ptr<mg::DisplayConfigurationPolicy>
 mir::DefaultServerConfiguration::the_display_configuration_policy()
 {
@@ -832,6 +976,77 @@ mir::DefaultServerConfiguration::the_display_configuration_policy()
         });
 }
 
+auto mir::DefaultServerConfiguration::the_host_connection()
+-> std::shared_ptr<graphics::nested::HostConnection>
+{
+    return host_connection(
+        [this]() -> std::shared_ptr<graphics::nested::HostConnection>
+        {
+            auto const options = the_options();
+
+            if (!options->is_set(standalone_opt))
+            {
+                if (!options->is_set(host_socket_opt))
+                    BOOST_THROW_EXCEPTION(mir::AbnormalExit("Exiting Mir! Specify either $MIR_SOCKET or --standalone"));
+
+                auto host_socket = options->get(host_socket_opt, "");
+                auto server_socket = the_socket_file();
+
+                if (server_socket == host_socket)
+                    BOOST_THROW_EXCEPTION(mir::AbnormalExit("Exiting Mir! Reason: Nested Mir and Host Mir cannot use the same socket file to accept connections!"));
+
+                return std::make_shared<graphics::nested::HostConnection>(
+                    host_socket,
+                    server_socket);
+            }
+            else
+            {
+                BOOST_THROW_EXCEPTION(std::logic_error("can only use host connection in nested mode"));
+            }
+        });
+}
+
+auto mir::DefaultServerConfiguration::the_nested_input_relay()
+-> std::shared_ptr<mi::NestedInputRelay>
+{
+    return nested_input_relay([]{ return std::make_shared<mi::NestedInputRelay>(); });
+}
+
+
+std::shared_ptr<msh::BroadcastingSessionEventSink>
+mir::DefaultServerConfiguration::the_broadcasting_session_event_sink()
+{
+    return broadcasting_session_event_sink(
+        []
+        {
+            return std::make_shared<msh::BroadcastingSessionEventSink>();
+        });
+}
+
+auto mir::DefaultServerConfiguration::the_connector_report()
+    -> std::shared_ptr<mf::ConnectorReport>
+{
+    return connector_report([this]
+        () -> std::shared_ptr<mf::ConnectorReport>
+        {
+            auto opt = the_options()->get(connector_report_opt, off_opt_value);
+
+            if (opt == log_opt_value)
+            {
+                return std::make_shared<ml::ConnectorReport>(the_logger());
+            }
+            else if (opt == off_opt_value)
+            {
+                return std::make_shared<mf::NullConnectorReport>();
+            }
+            else
+            {
+                throw AbnormalExit(std::string("Invalid ") + connector_report_opt + " option: " + opt +
+                    " (valid options are: \"" + off_opt_value + "\" and \"" + log_opt_value + "\")");
+            }
+        });
+}
+
 namespace
 {
 mir::SharedLibrary const* load_library(std::string const& libname)
