@@ -24,6 +24,10 @@
 
 namespace mir
 {
+namespace protobuf
+{
+class EventSequence;
+}
 namespace frontend
 {
 namespace detail
@@ -35,8 +39,12 @@ class EventSender : public  mir::frontend::EventSink
 public:
     explicit EventSender(std::shared_ptr<MessageSender> const& socket_sender);
     void handle_event(MirEvent const& e);
+    void handle_lifecycle_event(MirLifecycleState state);
+    void handle_display_config_change(graphics::DisplayConfiguration const& config);
 
 private:
+    void send_event_sequence(protobuf::EventSequence&);
+
     std::shared_ptr<MessageSender> const sender;
 };
  
