@@ -19,23 +19,31 @@
 #ifndef MIR_FRONTEND_PROTOBUF_BUFFER_PACKER_H_
 #define MIR_FRONTEND_PROTOBUF_BUFFER_PACKER_H_
 
-#include "mir/compositor/buffer_ipc_packer.h"
+#include "mir/graphics/buffer_ipc_packer.h"
 #include "mir_protobuf.pb.h"
 
 namespace mir
 {
+namespace graphics
+{
+class DisplayConfiguration;
+}
 namespace frontend
 {
 namespace detail
 {
 
-class ProtobufBufferPacker : public compositor::BufferIPCPacker
+void pack_protobuf_display_configuration(protobuf::DisplayConfiguration& protobuf_config,
+                                         graphics::DisplayConfiguration const& display_config);
+
+class ProtobufBufferPacker : public graphics::BufferIPCPacker
 {
 public:
     ProtobufBufferPacker(protobuf::Buffer*);
     void pack_fd(int);
     void pack_data(int);
     void pack_stride(geometry::Stride);
+    void pack_flags(unsigned int);
 private:
     protobuf::Buffer* buffer_response;
 };
