@@ -21,30 +21,25 @@
 
 #include "mir/graphics/internal_client.h"
 #include <memory>
+#include <map>
 
 namespace mir
 {
-namespace frontend
-{
-class Surface;
-}
 namespace graphics
 {
 namespace android
 {
-
 class MirNativeWindow;
 class InternalClient : public mir::graphics::InternalClient
 {
 public:
     InternalClient();
     EGLNativeDisplayType egl_native_display();
-    EGLNativeWindowType egl_native_window(std::shared_ptr<frontend::Surface> const&);
+    EGLNativeWindowType egl_native_window(std::shared_ptr<InternalSurface> const&);
 
 private:
-    std::shared_ptr<MirNativeWindow> client_window;
+    std::map<std::shared_ptr<InternalSurface>, std::shared_ptr<MirNativeWindow>> client_windows;
 };
-
 }
 }
 }
