@@ -26,6 +26,7 @@ namespace mir
 namespace graphics
 {
 class Buffer;
+class NativeBuffer;
 
 namespace android
 {
@@ -34,8 +35,10 @@ class InterpreterResourceCache
 public:
     InterpreterResourceCache() {}
 
-    virtual void store_buffer(std::shared_ptr<graphics::Buffer>const& buffer, ANativeWindowBuffer* key) = 0;
+    virtual void store_buffer(std::shared_ptr<graphics::Buffer>const& buffer,
+                              std::shared_ptr<graphics::NativeBuffer> const& key) = 0;
     virtual std::shared_ptr<graphics::Buffer> retrieve_buffer(ANativeWindowBuffer* key) = 0;
+    virtual void update_native_fence(ANativeWindowBuffer* key, int fence) = 0;
 
 protected:
     virtual ~InterpreterResourceCache() {}
