@@ -21,7 +21,6 @@
 
 #include <cstdint>
 #include <memory>
-#include <unordered_map>
 
 namespace mir
 {
@@ -31,24 +30,19 @@ namespace gbm
 {
 
 class KMSOutput;
-class PageFlipper;
 
 class KMSOutputContainer
 {
 public:
-    KMSOutputContainer(int drm_fd, std::shared_ptr<PageFlipper> const& page_flipper);
+    virtual ~KMSOutputContainer() = default;
 
-    std::shared_ptr<KMSOutput> get_kms_output_for(uint32_t connector_id);
+    virtual std::shared_ptr<KMSOutput> get_kms_output_for(uint32_t connector_id) = 0;
+    virtual void for_each_output(std::function<void(KMSOutput&)> functor) const = 0;
 
-    void for_each_output(std::function<void(KMSOutput&)> functor) const;
-
-private:
+protected:
+    KMSOutputContainer() = default;
     KMSOutputContainer(KMSOutputContainer const&) = delete;
     KMSOutputContainer& operator=(KMSOutputContainer const&) = delete;
-
-    int const drm_fd;
-    std::unordered_map<uint32_t,std::shared_ptr<KMSOutput>> outputs;
-    std::shared_ptr<PageFlipper> const page_flipper;
 };
 
 }
