@@ -23,11 +23,10 @@
 
 namespace mir
 {
-
-class MainLoop;
-
 namespace graphics
 {
+class EventHandlerRegister;
+
 namespace gbm
 {
 
@@ -38,7 +37,7 @@ public:
 
     virtual void set_graphics_mode() = 0;
     virtual void register_switch_handlers(
-        MainLoop& main_loop,
+        EventHandlerRegister& handlers,
         std::function<bool()> const& switch_away,
         std::function<bool()> const& switch_back) = 0;
 
