@@ -20,13 +20,14 @@
 #include "mir/surfaces/surface_stack_model.h"
 
 namespace ms = mir::surfaces;
+namespace msh = mir::shell;
 
 ms::SurfaceController::SurfaceController(std::shared_ptr<SurfaceStackModel> const& surface_stack) :
     surface_stack(surface_stack)
 {
 }
 
-std::weak_ptr<ms::Surface> ms::SurfaceController::create_surface(shell::SurfaceCreationParameters const& params)
+std::weak_ptr<ms::Surface> ms::SurfaceController::create_surface(msh::Session*, shell::SurfaceCreationParameters const& params)
 {
     return surface_stack->create_surface(params);
 }
