@@ -19,9 +19,10 @@
 #include "mir_test_framework/display_server_test_fixture.h"
 
 #include "mir_toolkit/mir_client_library.h"
+#include "mir_toolkit/mir_client_library_debug.h"
 #include "src/client/client_buffer.h"
 
-#include "mir/frontend/communicator.h"
+#include "mir/frontend/connector.h"
 
 #include "mir_protobuf.pb.h"
 
@@ -30,6 +31,12 @@
 #include <thread>
 #include <cstring>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+
+#include <errno.h>
+
 namespace mf = mir::frontend;
 namespace mc = mir::compositor;
 namespace mcl = mir::client;
@@ -161,7 +168,8 @@ TEST_F(DefaultDisplayServerTestFixture, client_library_creates_surface)
                 __PRETTY_FUNCTION__,
                 640, 480,
                 mir_pixel_format_abgr_8888,
-                mir_buffer_usage_hardware
+                mir_buffer_usage_hardware,
+                mir_display_output_id_invalid
             };
 
             mir_wait_for(mir_connection_create_surface(connection, &request_params, create_surface_callback, this));
@@ -223,7 +231,8 @@ TEST_F(DefaultDisplayServerTestFixture, surface_types)
                 __PRETTY_FUNCTION__,
                 640, 480,
                 mir_pixel_format_abgr_8888,
-                mir_buffer_usage_hardware
+                mir_buffer_usage_hardware,
+                mir_display_output_id_invalid
             };
 
             mir_wait_for(mir_connection_create_surface(connection, &request_params, create_surface_callback, this));
@@ -295,7 +304,8 @@ TEST_F(DefaultDisplayServerTestFixture, client_can_set_surface_state)
                 __PRETTY_FUNCTION__,
                 640, 480,
                 mir_pixel_format_abgr_8888,
-                mir_buffer_usage_hardware
+                mir_buffer_usage_hardware,
+                mir_display_output_id_invalid
             };
 
             surface = mir_connection_create_surface_sync(connection,
@@ -369,7 +379,8 @@ TEST_F(DefaultDisplayServerTestFixture, client_receives_surface_state_events)
                 __PRETTY_FUNCTION__,
                 640, 480,
                 mir_pixel_format_abgr_8888,
-                mir_buffer_usage_hardware
+                mir_buffer_usage_hardware,
+                mir_display_output_id_invalid
             };
 
             memset(&last_event, 0, sizeof last_event);
@@ -389,7 +400,7 @@ TEST_F(DefaultDisplayServerTestFixture, client_receives_surface_state_events)
 
             mir_surface_set_event_handler(surface, &delegate);
 
-            int surface_id = mir_surface_get_id(surface);
+            int surface_id = mir_debug_surface_id(surface);
 
             mir_wait_for(mir_surface_set_state(surface,
                                                mir_surface_state_fullscreen));
@@ -445,6 +456,83 @@ TEST_F(DefaultDisplayServerTestFixture, client_receives_surface_state_events)
     launch_client_process(client_config);
 }
 
+#ifndef ANDROID
+TEST_F(DefaultDisplayServerTestFixture, surface_scanout_flag_toggles)
+{
+    struct ClientConfig : ClientConfigCommon
+    {
+        void exec()
+        {
+            connection = mir_connect_sync(mir_test_socket, __PRETTY_FUNCTION__);
+            ASSERT_TRUE(connection != NULL);
+            EXPECT_TRUE(mir_connection_is_valid(connection));
+            EXPECT_STREQ(mir_connection_get_error_message(connection), "");
+
+            MirSurfaceParameters parm =
+            {
+                __PRETTY_FUNCTION__,
+                1280, 1024,
+                mir_pixel_format_abgr_8888,
+                mir_buffer_usage_hardware,
+                mir_display_output_id_invalid
+            };
+
+            surface = mir_connection_create_surface_sync(connection, &parm);
+            ASSERT_TRUE(mir_surface_is_valid(surface));
+            MirNativeBuffer *native;
+            mir_surface_get_current_buffer(surface, &native);
+            EXPECT_TRUE(native->flags & mir_buffer_flag_can_scanout);
+            mir_surface_swap_buffers_sync(surface);
+            EXPECT_TRUE(native->flags & mir_buffer_flag_can_scanout);
+            mir_surface_release_sync(surface);
+
+            parm.width = 100;
+            parm.height = 100;
+
+            surface = mir_connection_create_surface_sync(connection, &parm);
+            ASSERT_TRUE(mir_surface_is_valid(surface));
+            mir_surface_get_current_buffer(surface, &native);
+            EXPECT_FALSE(native->flags & mir_buffer_flag_can_scanout);
+            mir_surface_swap_buffers_sync(surface);
+            EXPECT_FALSE(native->flags & mir_buffer_flag_can_scanout);
+            mir_surface_release_sync(surface);
+
+            parm.width = 800;
+            parm.height = 600;
+            parm.buffer_usage = mir_buffer_usage_software;
+
+            surface = mir_connection_create_surface_sync(connection, &parm);
+            ASSERT_TRUE(mir_surface_is_valid(surface));
+            mir_surface_get_current_buffer(surface, &native);
+            EXPECT_FALSE(native->flags & mir_buffer_flag_can_scanout);
+            mir_surface_swap_buffers_sync(surface);
+            EXPECT_FALSE(native->flags & mir_buffer_flag_can_scanout);
+            mir_surface_release_sync(surface);
+
+            parm.buffer_usage = mir_buffer_usage_hardware;
+
+            surface = mir_connection_create_surface_sync(connection, &parm);
+            ASSERT_TRUE(mir_surface_is_valid(surface));
+            mir_surface_get_current_buffer(surface, &native);
+            EXPECT_TRUE(native->flags & mir_buffer_flag_can_scanout);
+            mir_surface_swap_buffers_sync(surface);
+            EXPECT_TRUE(native->flags & mir_buffer_flag_can_scanout);
+            mir_surface_release_sync(surface);
+
+            mir_connection_release(connection);
+        }
+
+        // this test relies on gbm drivers, use real graphics always
+        bool use_real_graphics(mir::options::Option const&) override
+        {
+            return true;
+        }
+    } client_config;
+
+    launch_client_process(client_config);
+}
+#endif
+
 TEST_F(DefaultDisplayServerTestFixture, client_library_creates_multiple_surfaces)
 {
     int const n_surfaces = 13;
@@ -492,7 +580,8 @@ TEST_F(DefaultDisplayServerTestFixture, client_library_creates_multiple_surfaces
                     __PRETTY_FUNCTION__,
                     640, 480,
                     mir_pixel_format_abgr_8888,
-                    mir_buffer_usage_hardware
+                    mir_buffer_usage_hardware,
+                    mir_display_output_id_invalid
                 };
 
                 mir_wait_for(mir_connection_create_surface(connection, &request_params, create_surface_callback, this));
@@ -540,7 +629,8 @@ TEST_F(DefaultDisplayServerTestFixture, client_library_accesses_and_advances_buf
                 __PRETTY_FUNCTION__,
                 640, 480,
                 mir_pixel_format_abgr_8888,
-                mir_buffer_usage_hardware
+                mir_buffer_usage_hardware,
+                mir_display_output_id_invalid
             };
 
             mir_wait_for(mir_connection_create_surface(connection, &request_params, create_surface_callback, this));
@@ -579,7 +669,8 @@ TEST_F(DefaultDisplayServerTestFixture, fully_synchronous_client)
                 __PRETTY_FUNCTION__,
                 640, 480,
                 mir_pixel_format_abgr_8888,
-                mir_buffer_usage_software
+                mir_buffer_usage_software,
+                mir_display_output_id_invalid
             };
 
             surface = mir_connection_create_surface_sync(connection, &request_params);
@@ -641,7 +732,8 @@ TEST_F(DefaultDisplayServerTestFixture, highly_threaded_client)
                 __PRETTY_FUNCTION__,
                 640, 480,
                 mir_pixel_format_abgr_8888,
-                mir_buffer_usage_software
+                mir_buffer_usage_software,
+                mir_display_output_id_invalid
             };
 
             surface = mir_connection_create_surface_sync(connection,
@@ -709,11 +801,11 @@ TEST_F(DefaultDisplayServerTestFixture, client_library_accesses_display_info)
 
             auto configuration = mir_connection_create_display_config(connection);
             ASSERT_NE(nullptr, configuration);
-            ASSERT_GT(configuration->num_displays, 0u);
-            ASSERT_NE(nullptr, configuration->displays);
-            for (auto i=0u; i < configuration->num_displays; i++)
+            ASSERT_GT(configuration->num_outputs, 0u);
+            ASSERT_NE(nullptr, configuration->outputs);
+            for (auto i=0u; i < configuration->num_outputs; i++)
             {
-                MirDisplayOutput* disp = &configuration->displays[i];
+                MirDisplayOutput* disp = &configuration->outputs[i];
                 ASSERT_NE(nullptr, disp); 
                 EXPECT_GE(disp->num_modes, disp->current_mode);
                 EXPECT_GE(disp->num_output_formats, disp->current_output_format);
@@ -745,6 +837,13 @@ TEST_F(DefaultDisplayServerTestFixture, connect_errors_handled)
                 FAIL() << error;
             }
         }
+
+        //we are testing the connect function itself, without getting to the
+        // point where drivers are used, so force using production config
+        bool use_real_graphics(mir::options::Option const&) override
+        {
+            return true;
+        }
     } client_config;
 
     launch_client_process(client_config);
@@ -763,7 +862,8 @@ TEST_F(DefaultDisplayServerTestFixture, connect_errors_dont_blow_up)
                 __PRETTY_FUNCTION__,
                 640, 480,
                 mir_pixel_format_abgr_8888,
-                mir_buffer_usage_hardware
+                mir_buffer_usage_hardware,
+                mir_display_output_id_invalid
             };
 
             mir_wait_for(mir_connection_create_surface(connection, &request_params, create_surface_callback, this));
@@ -773,6 +873,13 @@ TEST_F(DefaultDisplayServerTestFixture, connect_errors_dont_blow_up)
 
             mir_connection_release(connection);
         }
+
+        //we are testing the connect function itself, without getting to the
+        // point where drivers are used, so force using production config
+        bool use_real_graphics(mir::options::Option const&) override
+        {
+            return true;
+        }
     } client_config;
 
     launch_client_process(client_config);
@@ -818,7 +925,8 @@ TEST_F(DefaultDisplayServerTestFixture, ClientLibraryThreadsHandleNoSignals)
                 __PRETTY_FUNCTION__,
                 640, 480,
                 mir_pixel_format_abgr_8888,
-                mir_buffer_usage_software
+                mir_buffer_usage_software,
+                mir_display_output_id_invalid
             };
 
             surface = mir_connection_create_surface_sync(conn, &request_params);
@@ -866,4 +974,76 @@ TEST_F(DefaultDisplayServerTestFixture, ClientLibraryDoesNotInterfereWithClientS
     launch_client_process(client_config);
 }
 
+TEST_F(DefaultDisplayServerTestFixture, MultiSurfaceClientTracksBufferFdsCorrectly)
+{
+    struct ClientConfig : ClientConfigCommon
+    {
+        void exec()
+        {
+
+            mir_wait_for(mir_connect(mir_test_socket, __PRETTY_FUNCTION__, connection_callback, this));
+
+            ASSERT_TRUE(connection != NULL);
+            EXPECT_TRUE(mir_connection_is_valid(connection));
+            EXPECT_STREQ("", mir_connection_get_error_message(connection));
+
+            MirSurfaceParameters const request_params =
+            {
+                __PRETTY_FUNCTION__,
+                640, 480,
+                mir_pixel_format_abgr_8888,
+                mir_buffer_usage_hardware,
+                mir_display_output_id_invalid
+            };
+
+            surf_one = mir_connection_create_surface_sync(connection, &request_params);
+            surf_two = mir_connection_create_surface_sync(connection, &request_params);
+
+            ASSERT_TRUE(surf_one != NULL);
+            ASSERT_TRUE(surf_two != NULL);
+
+            buffers = 0;
+
+            while (buffers < 1024)
+            {
+                mir_surface_swap_buffers_sync(surf_one);
+                mir_surface_swap_buffers_sync(surf_two);
+
+                buffers++;
+            }
+
+            /* We should not have any stray fds hanging around.
+               Test this by trying to open a new one */
+            int canary_fd;
+            canary_fd = open("/dev/null", O_RDONLY);
+
+            ASSERT_TRUE(canary_fd > 0) << "Failed to open canary file descriptor: "<< strerror(errno);
+            EXPECT_TRUE(canary_fd < 1024);
+
+            close(canary_fd);
+
+            mir_wait_for(mir_surface_release(surf_one, release_surface_callback, this));
+            mir_wait_for(mir_surface_release(surf_two, release_surface_callback, this));
+
+            ASSERT_TRUE(surf_one == NULL);
+            ASSERT_TRUE(surf_two == NULL);
+
+            mir_connection_release(connection);
+        }
+
+        virtual void surface_released (MirSurface* surf)
+        {
+            if (surf == surf_one)
+                surf_one = NULL;
+            if (surf == surf_two)
+                surf_two = NULL;
+        }
+
+        MirSurface* surf_one;
+        MirSurface* surf_two;
+    } client_config;
+
+    launch_client_process(client_config);
+}
+
 }
