@@ -50,8 +50,8 @@ mgg::FakeDRMResources::FakeDRMResources()
     uint32_t const connector1_id{31};
     uint32_t const all_crtcs_mask{0x3};
 
-    modes.push_back(create_mode(1920, 1080, 138500, 2080, 1111));
-    modes.push_back(create_mode(832, 624, 57284, 1152, 667));
+    modes.push_back(create_mode(1920, 1080, 138500, 2080, 1111, PreferredMode));
+    modes.push_back(create_mode(832, 624, 57284, 1152, 667, NormalMode));
     connector_encoder_ids.push_back(encoder0_id);
     connector_encoder_ids.push_back(encoder1_id);
 
@@ -61,9 +61,11 @@ mgg::FakeDRMResources::FakeDRMResources()
     add_encoder(encoder0_id, invalid_id, all_crtcs_mask);
     add_encoder(encoder1_id, crtc1_id, all_crtcs_mask);
 
-    add_connector(connector0_id, DRM_MODE_DISCONNECTED, invalid_id,
+    add_connector(connector0_id, DRM_MODE_CONNECTOR_VGA,
+                  DRM_MODE_DISCONNECTED, invalid_id,
                   modes_empty, connector_encoder_ids, geom::Size());
-    add_connector(connector1_id, DRM_MODE_CONNECTED, encoder1_id,
+    add_connector(connector1_id, DRM_MODE_CONNECTOR_DVID,
+                  DRM_MODE_CONNECTED, encoder1_id,
                   modes, connector_encoder_ids,
                   geom::Size{121, 144});
 
@@ -148,6 +150,7 @@ void mgg::FakeDRMResources::add_encoder(uint32_t encoder_id, uint32_t crtc_id,
 }
 
 void mgg::FakeDRMResources::add_connector(uint32_t connector_id,
+                                          uint32_t type,
                                           drmModeConnection connection,
                                           uint32_t encoder_id,
                                           std::vector<drmModeModeInfo>& modes,
@@ -157,6 +160,7 @@ void mgg::FakeDRMResources::add_connector(uint32_t connector_id,
     drmModeConnector connector = drmModeConnector();
 
     connector.connector_id = connector_id;
+    connector.connector_type = type;
     connector.connection = connection;
     connector.encoder_id = encoder_id;
     connector.modes = modes.data();
@@ -202,7 +206,8 @@ drmModeConnector* mgg::FakeDRMResources::find_connector(uint32_t id)
 
 drmModeModeInfo mgg::FakeDRMResources::create_mode(uint16_t hdisplay, uint16_t vdisplay,
                                                    uint32_t clock, uint16_t htotal,
-                                                   uint16_t vtotal)
+                                                   uint16_t vtotal,
+                                                   ModePreference preferred)
 {
     drmModeModeInfo mode = drmModeModeInfo();
 
@@ -211,6 +216,8 @@ drmModeModeInfo mgg::FakeDRMResources::create_mode(uint16_t hdisplay, uint16_t v
     mode.clock = clock;
     mode.htotal = htotal;
     mode.vtotal = vtotal;
+    if (preferred)
+        mode.type |= DRM_MODE_TYPE_PREFERRED;
 
     return mode;
 }
