@@ -25,6 +25,7 @@
 #include "mir_test_doubles/mock_hwc_interface.h"
 #include "mir_test_doubles/mock_display_report.h"
 #include "mir_test_doubles/mock_egl.h"
+#include "mir_test_doubles/stub_display_support_provider.h"
 
 #include <gtest/gtest.h>
 #include <memory>
@@ -91,7 +92,8 @@ std::shared_ptr<mg::Display> make_display<DummyGPUDisplayType>(
     std::shared_ptr<mg::DisplayReport> const& listener)
 {
     auto db_factory = std::make_shared<mga::GPUAndroidDisplayBufferFactory>();
-    return std::make_shared<mga::AndroidDisplay>(fbwin, db_factory, listener);
+    return std::make_shared<mga::AndroidDisplay>(fbwin, db_factory, 
+        std::make_shared<mtd::StubDisplaySupportProvider>(), listener);
 }
 
 template <>
@@ -103,7 +105,8 @@ std::shared_ptr<mg::Display> make_display<DummyHWCDisplayType>(
     ON_CALL(*mock_hwc_device, display_size())
         .WillByDefault(testing::Return(display_size)); 
     auto db_factory = std::make_shared<mga::HWCAndroidDisplayBufferFactory>(mock_hwc_device);
-    return std::make_shared<mga::AndroidDisplay>(fbwin, db_factory, listener);
+    return std::make_shared<mga::AndroidDisplay>(fbwin, db_factory,
+        std::make_shared<mtd::StubDisplaySupportProvider>(), listener);
 }
 
 
@@ -622,7 +625,7 @@ TYPED_TEST(AndroidTestFramebufferInit, android_display_configuration_info)
     EXPECT_EQ(display_size, disp_mode.size);
     //TODO fill refresh rate accordingly
 
-    EXPECT_EQ(mg::DisplayConfigurationOutputId{0}, disp_conf.id); 
+    EXPECT_EQ(mg::DisplayConfigurationOutputId{1}, disp_conf.id); 
     EXPECT_EQ(mg::DisplayConfigurationCardId{0}, disp_conf.card_id); 
     EXPECT_TRUE(disp_conf.connected);
     EXPECT_TRUE(disp_conf.used);
@@ -631,15 +634,3 @@ TYPED_TEST(AndroidTestFramebufferInit, android_display_configuration_info)
     EXPECT_EQ(0, disp_conf.current_mode_index);
     //TODO fill physical_size_mm fields accordingly;
 }
-
-//Cannot configure android displays at this time
-TYPED_TEST(AndroidTestFramebufferInit, android_display_configure_output)
-{
-    auto display = make_display<TypeParam>(this->native_win, this->mock_display_report);
-    auto config = display->configuration();
-
-    mg::DisplayConfigurationOutputId id{0};
-    EXPECT_THROW({
-        config->configure_output(id, true, geom::Point{0,0}, 0);
-    }, std::runtime_error);
-}
