@@ -64,7 +64,7 @@ public:
 
     MOCK_METHOD0(set_graphics_mode, void());
     MOCK_METHOD3(register_switch_handlers,
-                 void(mir::MainLoop&,
+                 void(mg::EventHandlerRegister&,
                       std::function<bool()> const&,
                       std::function<bool()> const&));
 };
@@ -75,7 +75,7 @@ public:
     ~MockVideoDevices() noexcept(true) {}
 
     MOCK_METHOD2(register_change_handler,
-                 void(mir::MainLoop&,
+                 void(mg::EventHandlerRegister&,
                       std::function<void()> const&));
 };
 
@@ -671,10 +671,10 @@ TEST_F(GBMDisplayTest, set_or_drop_drm_master_failure_throws_and_reports_error)
     using namespace testing;
 
     EXPECT_CALL(mock_drm, drmDropMaster(_))
-        .WillOnce(SetErrnoAndReturn(EACCES, -1));
+        .WillOnce(SetErrnoAndReturn(EACCES, -EACCES));
 
     EXPECT_CALL(mock_drm, drmSetMaster(_))
-        .WillOnce(SetErrnoAndReturn(EPERM, -1));
+        .WillOnce(SetErrnoAndReturn(EPERM, -EPERM));
 
     EXPECT_CALL(*mock_report, report_drm_master_failure(EACCES));
     EXPECT_CALL(*mock_report, report_drm_master_failure(EPERM));
