@@ -21,7 +21,7 @@
 #include "mir/compositor/buffer_stream_surfaces.h"
 #include "mir/surfaces/buffer_stream_factory.h"
 #include "src/server/compositor/buffer_bundle.h"
-#include "mir/compositor/buffer_properties.h"
+#include "mir/graphics/buffer_properties.h"
 #include "mir/compositor/scene.h"
 #include "mir/geometry/rectangle.h"
 #include "mir/shell/surface_creation_parameters.h"
@@ -48,6 +48,8 @@
 
 #include <memory>
 #include <stdexcept>
+#include <thread>
+#include <atomic>
 
 namespace mc = mir::compositor;
 namespace mg = mir::graphics;
@@ -67,12 +69,15 @@ class NullBufferBundle : public mc::BufferBundle
 public:
     virtual std::shared_ptr<mg::Buffer> client_acquire() { return std::shared_ptr<mg::Buffer>(); }
     virtual void client_release(std::shared_ptr<mg::Buffer> const&) {}
-    virtual std::shared_ptr<mg::Buffer> compositor_acquire(){ return std::shared_ptr<mg::Buffer>(); };
+    virtual std::shared_ptr<mg::Buffer> compositor_acquire(unsigned long)
+        { return std::shared_ptr<mg::Buffer>(); };
     virtual void compositor_release(std::shared_ptr<mg::Buffer> const&){}
+    virtual std::shared_ptr<mg::Buffer> snapshot_acquire(){ return std::shared_ptr<mg::Buffer>(); };
+    virtual void snapshot_release(std::shared_ptr<mg::Buffer> const&){}
     virtual void force_client_abort() {}
     void force_requests_to_complete() {}
     virtual void allow_framedropping(bool) {}
-    virtual mc::BufferProperties properties() const { return mc::BufferProperties{}; };
+    virtual mg::BufferProperties properties() const { return mg::BufferProperties{}; };
 };
 
 
@@ -148,7 +153,7 @@ struct MockSurfaceAllocator : public ms::SurfaceFactory
 
 struct StubBufferStreamFactory : public ms::BufferStreamFactory
 {
-    std::shared_ptr<ms::BufferStream> create_buffer_stream(mc::BufferProperties const&)
+    std::shared_ptr<ms::BufferStream> create_buffer_stream(mg::BufferProperties const&)
     {
         return std::make_shared<mc::BufferStreamSurfaces>(
             std::make_shared<NullBufferBundle>());
@@ -464,3 +469,121 @@ TEST_F(SurfaceStack, raise_throw_behavior)
             stack.raise(null_surface);
     }, std::runtime_error);
 }
+
+namespace
+{
+
+struct UniqueOperatorForScene : public mc::OperatorForScene
+{
+    UniqueOperatorForScene(const char *&owner)
+        : owner(owner)
+    {
+    }
+
+    void operator()(const mc::CompositingCriteria &, ms::BufferStream&)
+    {
+        ASSERT_STREQ("", owner);
+        owner = "UniqueOperatorForScene";
+        std::this_thread::yield();
+        owner = "";
+    }
+
+    const char *&owner;
+};
+
+void tinker_scene(mc::Scene &scene,
+                  const char *&owner,
+                  const std::atomic_bool &done)
+{
+    while (!done.load())
+    {
+        std::this_thread::yield();
+
+        std::lock_guard<mc::Scene> lock(scene);
+        ASSERT_STREQ("", owner);
+        owner = "tinkerer";
+        std::this_thread::yield();
+        owner = "";
+    }
+}
+
+}
+
+TEST_F(SurfaceStack, is_locked_during_iteration)
+{
+    using namespace ::testing;
+
+    ms::SurfaceStack stack(mt::fake_shared(mock_surface_allocator),
+                           mt::fake_shared(input_registrar));
+
+    auto s1 = stack.create_surface(default_params);
+    auto criteria1 = s1.lock()->compositing_criteria();
+    auto stream1 = s1.lock()->buffer_stream();
+    auto s2 = stack.create_surface(default_params);
+    auto criteria2 = s2.lock()->compositing_criteria();
+    auto stream2 = s2.lock()->buffer_stream();
+    auto s3 = stack.create_surface(default_params);
+    auto criteria3 = s3.lock()->compositing_criteria();
+    auto stream3 = s3.lock()->buffer_stream();
+
+    MockFilterForScene filter;
+    Sequence seq1;
+    EXPECT_CALL(filter, filter(Ref(*criteria1)))
+        .InSequence(seq1)
+        .WillRepeatedly(Return(true));
+    EXPECT_CALL(filter, filter(Ref(*criteria2)))
+        .InSequence(seq1)
+        .WillRepeatedly(Return(false));
+    EXPECT_CALL(filter, filter(Ref(*criteria3)))
+        .InSequence(seq1)
+        .WillRepeatedly(Return(true));
+
+    const char *owner = "";
+    UniqueOperatorForScene op(owner);
+
+    std::atomic_bool done(false);
+    std::thread tinkerer(tinker_scene,
+        std::ref(stack), std::ref(owner), std::ref(done));
+
+    for (int i = 0; i < 1000; i++)
+    {
+        stack.lock();
+        ASSERT_STREQ("", owner);
+        owner = "main_before";
+        std::this_thread::yield();
+        owner = "";
+        stack.unlock();
+
+        stack.for_each_if(filter, op);
+
+        stack.lock();
+        ASSERT_STREQ("", owner);
+        owner = "main_after";
+        std::this_thread::yield();
+        owner = "";
+        stack.unlock();
+    }
+
+    done = true;
+    tinkerer.join();
+}
+
+TEST_F(SurfaceStack, is_recursively_lockable)
+{
+    ms::SurfaceStack stack(mt::fake_shared(mock_surface_allocator),
+                           mt::fake_shared(input_registrar));
+
+    StubFilterForScene filter;
+    StubOperatorForScene op;
+
+    stack.lock();
+    stack.for_each_if(filter, op);
+    stack.unlock();
+
+    stack.lock();
+    stack.lock();
+    stack.lock();
+    stack.unlock();
+    stack.unlock();
+    stack.unlock();
+}
