@@ -21,26 +21,26 @@ pushd ${BUILD_DIR} > /dev/null
     # Upload and run the tests!
     # Requires: https://wiki.canonical.com/ProductStrategyTeam/Android/Deploy
     #
-    RUN_DIR=/data/mirtest
+    RUN_DIR=/tmp/mirtest
 
     adb wait-for-device
     adb root
     adb wait-for-device
-    adb shell stop
     adb shell mkdir -p ${RUN_DIR}
 
     for x in bin/acceptance-tests \
              bin/integration-tests \
              bin/unit-tests \
-             lib/libmirclient.so.0 \
-             lib/libmirprotobuf.so.0 \
-             lib/libmirserver.so.0
+             lib/libmirclient.so.* \
+             lib/libmirprotobuf.so.* \
+             lib/libmirplatform.so \
+             lib/libmirplatformgraphics.so \
+             lib/libmirserver.so.*
     do
         adb push $x ${RUN_DIR}
     done
 
-    echo "ubuntu_chroot shell;
-        cd ${RUN_DIR};
+    echo "cd ${RUN_DIR};
         export GTEST_OUTPUT=xml:./;
         export LD_LIBRARY_PATH=.;
         ./unit-tests;
